from django.conf.urls import include, url

urlpatterns = [
    url(r'^autocomplete/', include('autocomplete_light.urls')),
    url('^blog/', include('info.urls.blog')),
    url('^info/', include('info.urls.pages')),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime
import markitup.fields


class Migration(migrations.Migration):

    dependencies = [
        ('file_archive', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, auto_now_add=True)),
                ('updated', models.DateTimeField(default=datetime.datetime.now, auto_now=True)),
                ('name', models.CharField(unique=True, max_length=300)),
                ('slug', models.SlugField(unique=True)),
                ('summary', markitup.fields.MarkupField(no_rendered_field=True, blank=True)),
                ('_summary_rendered', models.TextField(editable=False, blank=True)),
            ],
            options={
                'verbose_name_plural': 'categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InfoPage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, auto_now_add=True)),
                ('updated', models.DateTimeField(default=datetime.datetime.now, auto_now=True)),
                ('title', models.CharField(unique=True, max_length=300)),
                ('slug', models.SlugField(unique=True)),
                ('markdown_content', markitup.fields.MarkupField(default=b'', help_text=b'When linking to other pages use their slugs as the address (note that these links do not work in the preview, but will on the real site)', no_rendered_field=True, blank=True)),
                ('raw_content', models.TextField(default=b'', help_text=b"You can enter raw HTML into this box, and it will be used if 'Enter content as raw HTML' is selected", verbose_name=b'Raw HTML', blank=True)),
                ('use_raw', models.BooleanField(default=False, verbose_name=b'Enter content as raw HTML')),
                ('kind', models.CharField(default=b'page', max_length=10, choices=[(b'blog', b'Blog'), (b'page', b'Page')])),
                ('publication_date', models.DateTimeField(default=datetime.datetime.now)),
                ('_markdown_content_rendered', models.TextField(editable=False, blank=True)),
                ('categories', models.ManyToManyField(related_name='entries', to='info.Category', blank=True)),
                ('featured_image_file', models.ForeignKey(blank=True, to='file_archive.File', null=True)),
            ],
            options={
                'ordering': ['title'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(default=datetime.datetime.now, auto_now_add=True)),
                ('updated', models.DateTimeField(default=datetime.datetime.now, auto_now=True)),
                ('name', models.CharField(unique=True, max_length=300)),
                ('slug', models.SlugField(unique=True)),
            ],
            options={
                'ordering': ('name',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ViewCount',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField()),
                ('count', models.IntegerField()),
                ('page', models.ForeignKey(to='info.InfoPage')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='viewcount',
            unique_together=set([('date', 'page')]),
        ),
        migrations.AddField(
            model_name='infopage',
            name='tags',
            field=models.ManyToManyField(related_name='entries', to='info.Tag', blank=True),
            preserve_default=True,
        ),
    ]



from django.conf.urls import patterns, url, handler404

from ..views import InfoBlogView, InfoBlogList, InfoBlogFeed, InfoBlogCategory, InfoBlogTag


urlpatterns = patterns('',
    url(r'^$', InfoBlogList.as_view(), name='info_blog_list'),
    url(r'^category/(?P<slug>[\w\-,]+)$', InfoBlogCategory.as_view(), name='info_blog_category'),
    url(r'^tag/(?P<slug>[\w\-,]+)$',      InfoBlogTag.as_view(),      name='info_blog_tag'),
    url(r'^feed\.rss$', InfoBlogFeed(), name='info_blog_feed'),
    url(r'^(?P<slug>[\w\-]+)$', InfoBlogView.as_view(), name='info_blog'),
    url(r'^.*$', handler404),
)



import datetime
import lxml
from lxml.html.clean import Cleaner
import re

from django.conf import settings
from django.db import models
from django.core.urlresolvers import reverse
from django.utils.text import slugify

from markitup.fields import MarkupField

from file_archive.models import File

class ModelBase(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class LabelModelBase(ModelBase):

    """
    The tags and categories are essentially the same thing in the database. Use
    a common model for most of the fields etc.
    """

    name = models.CharField(max_length=300, unique=True)
    slug = models.SlugField(unique=True)

    def __unicode__(self):
        return self.name

    class Meta():
        abstract = True
        ordering = ( 'name', )


class Category(LabelModelBase):

    summary = MarkupField(blank=True)

    class Meta():
        verbose_name_plural = 'categories'


class Tag(LabelModelBase):
    pass


class InfoPage(ModelBase):
    """
    InfoPage - store static pages in the database so they can be edited in the
    admin. Also simple blog posts.

    There are several pages on a site that are static - ie they don't change
    very often. However sometimes they need to change and it is conveniant to do
    this via the admin, rather than editing the html on disk.

    This module allows you to do that.

    Each page has a slug - which is used to identify it in the url. So for
    example if you had a site FAQ the slug might be 'faq' and its url would
    become something like http://example.com/info/faq - where 'info' is where
    these pages are stored.

    Pages also have titles - which are shown at the top of the page.

    Both slugs and titles must be unique to each page.

    The content of the page is formatted using 'markdown' - which allows you to
    include bulleted lists, headings, styling and links.

    The page with the slug 'index' is special - it is used as the index page to
    all the other info pages, and so should probably be a table of contents or
    similar.

    Pages can also be marked as 'blog' in which case they are presented in
    newest first order on the '/blog' page, and on their own blog page.
    """

    title   = models.CharField(max_length=300, unique=True)
    slug    = models.SlugField(unique=True)
    markdown_content = MarkupField(
        blank=True,
        default='',
        help_text="When linking to other pages use their slugs as the address (note that these links do not work in the preview, but will on the real site)",
    )

    raw_content = models.TextField(
        "Raw HTML",
        blank=True,
        default='',
        help_text="You can enter raw HTML into this box, and it will be used if 'Enter content as raw HTML' is selected"
    )
    use_raw = models.BooleanField(
        'Enter content as raw HTML',
        default=False,
    )

    featured_image_file = models.ForeignKey(File, blank=True, null=True)

    KIND_PAGE = 'page'
    KIND_BLOG = 'blog'
    KIND_CHOICES = (
        (KIND_BLOG, 'Blog'),
        (KIND_PAGE, 'Page')
    )
    kind = models.CharField(max_length=10, choices=KIND_CHOICES, default=KIND_PAGE)

    # When was this page/post published. Could use updated or created but it
    # makes sense to make this seperate now as it will facilitate queing up
    # posts to be published in future easier.
    publication_date = models.DateTimeField( default=datetime.datetime.now )

    # Link to the categories and tags, use a custom related_name as this model
    # can represent both pages and posts.
    categories = models.ManyToManyField(Category, related_name="entries", blank=True)
    tags       = models.ManyToManyField(Tag,      related_name="entries", blank=True)


    def __unicode__(self):
        return self.title

    def css_class(self):
        return self._meta.model_name

    def name(self):
        return str(self)

    def _clean_html(self, html):
        if not html:
            return ''
        cleaner = Cleaner(style=True, scripts=True)
        return cleaner.clean_html(html)

    @property
    def content_as_html(self):
        if settings.INFO_PAGES_ALLOW_RAW_HTML and self.use_raw:
            # Parsing the HTML with lxml and outputting it again
            # should ensure that we have only well-formed HTML:
            parsed = lxml.html.fromstring(self.raw_content)
            return lxml.etree.tostring(parsed, method='html')
        else:
            # Since there seems to be some doubt about whether
            # markdown's safe_mode is really safe, clean the rendered
            # HTML to remove any potentially dangerous tags first
            return self._clean_html(self.markdown_content.rendered or '')

    @property
    def content_as_cleaned_html(self):
        return self._clean_html(self.content_as_html)

    @property
    def content_as_plain_text(self):
        cleaned_html = self.content_as_cleaned_html
        cleaned_text = lxml.html.fromstring(cleaned_html).text_content()
        return re.sub(r'(?ms)\s+', ' ', cleaned_text).strip()

    def content_with_anchors(self):
        """ Returns content with an anchor tag <a> inserted above every heading element
            (the anchor name is the slugified heading text). For example:
            <h2>Halt! Who goes there?"</h2>
            becomes
            <a name="halt-who-goes-there">
            <h2>Halt! Who goes there?"</h2>"""
        def prepend_anchor_tag( match ):
            return '<a name="%s"></a>%s%s' % (slugify(match.group(2)), match.group(1), match.group(2))
        headings_regexp = re.compile( '(<h\d+[^>]*>)([^<]*)')
        return headings_regexp.sub( prepend_anchor_tag, self.content_as_html)

    @models.permalink
    def get_absolute_url(self):

        if self.kind == self.KIND_PAGE:
            url_name = 'info_page'
        elif self.kind == self.KIND_BLOG:
            url_name = 'info_blog'
        else:
            raise Exception("Unexpected kind '{0}'".format(self.kind))

        return ( url_name, [ self.slug ] )

    def get_admin_url(self):
        url = reverse(
            'admin:%s_%s_change' % ( self._meta.app_label, self._meta.model_name),
            args=[self.id]
        )
        return url

    class Meta:
        ordering = ['title']


class ViewCount(models.Model):
    date = models.DateField()
    page = models.ForeignKey(InfoPage)

    # Keeping a normal integer here just in case we at some point want
    # to manually insert a negative count to keep a post down.
    count = models.IntegerField()

    class Meta:
        unique_together = ('date', 'page')



from datetime import date, timedelta
import itertools
import mock
from os.path import dirname, join
import re

from django.core.files.storage import FileSystemStorage
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.test import TestCase, Client

from nose.plugins.attrib import attr

from file_archive.models import File
from .migration_helpers import get_first_image_file
from .models import InfoPage, ViewCount


class InfoTest(TestCase):

    def setUp(self):
        pass

    def test_get_absolute_url(self):
        page = InfoPage(slug="page", title="Page Title", markdown_content="blah", kind=InfoPage.KIND_PAGE)
        post = InfoPage(slug="post", title="Post Title", markdown_content="blah", kind=InfoPage.KIND_BLOG)

        self.assertEqual(page.get_absolute_url(), "/info/page")
        self.assertEqual(post.get_absolute_url(), "/blog/post")

    def test_empty_info_page(self):
        empty_post = InfoPage.objects.create(
            slug='empty-of-info',
            title='A page with no content',
            markdown_content='',
            use_raw=False,
            kind=InfoPage.KIND_BLOG
        )
        # At one point with empty markdown_content this would cause an
        # exception, so add this to check for any regression:
        self.assertEqual(empty_post.content_as_html, '')

    @override_settings(INFO_PAGES_ALLOW_RAW_HTML=True)
    def test_blog_raw_html(self):
        danger_post = InfoPage.objects.create(
            slug="danger",
            title="Visualization",
            raw_content='''<h1 class="foo">Hello there</h1>
                <script>alert('hi!');</script>
                <p>blah blah, unclosed paragraph
                <iframe></iframe>
                <div>And then a div...</div>''',
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )
        try:
            # For reasons I don't understand, on Travis the result of
            # this has no space between '</script>' and '<p>' but not locally...
            as_html = re.sub(r'(?ms)\s+', ' ', danger_post.content_as_html)
            as_html = re.sub(r'</script><p>', '</script> <p>', as_html)
            self.assertEqual(
                as_html,
                "<div><h1 class=\"foo\">Hello there</h1> <script>alert('hi!');</script> <p>blah blah, unclosed paragraph <iframe></iframe> </p><div>And then a div...</div></div>"
            )
            self.assertEqual(
                re.sub(r'(?ms)\s+', ' ', danger_post.content_as_cleaned_html),
                "<div><h1 class=\"foo\">Hello there</h1> <p>blah blah, unclosed paragraph </p><div>And then a div...</div></div>"
            )
            self.assertEqual(
                re.sub(r'(?ms)\s+', ' ', danger_post.content_as_plain_text),
                "Hello there blah blah, unclosed paragraph And then a div..."
            )
        finally:
            danger_post.delete()

    @override_settings(INFO_PAGES_ALLOW_RAW_HTML=True)
    def test_no_self_closing_tags(self):
        example_post = InfoPage.objects.create(
            slug="post-with-video",
            title="Embedded YouTube Video",
            raw_content='''<h1 class="foo">Hello there</h1>
                <p>An introductory paragraph:
                <iframe></iframe>
                <p>And then a trailing paragraph...</p>''',
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )
        self.assertEqual(
            example_post.content_as_html,
            '''<div><h1 class="foo">Hello there</h1>
                <p>An introductory paragraph:
                <iframe></iframe>
                </p><p>And then a trailing paragraph...</p></div>'''
        )


@override_settings(INFO_PAGES_ALLOW_RAW_HTML=True)
class ViewCountsTest(TestCase):

    @mock.patch('info.views.date')
    def test_blog_post_views_are_counted(self, mockdate=None):
        example_post_1 = InfoPage.objects.create(
            slug="post",
            title="Example title",
            raw_content="Example content",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )

        example_post_2 = InfoPage.objects.create(
            slug="post2",
            title="Example title 2",
            raw_content="Example content 2",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )

        post_1_url = reverse('info_blog', kwargs={'slug': 'post'})
        post_2_url = reverse('info_blog', kwargs={'slug': 'post2'})

        day1 = date(2015, 01, 01)
        day2 = date(2015, 01, 02)

        mockdate.today.return_value = day1

        # The view counts table should be empty.
        self.assertFalse(ViewCount.objects.exists())

        # View the first post once
        self.client.get(post_1_url)

        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day1).count, 1)
        self.assertFalse(ViewCount.objects.filter(page=example_post_2.id).exists())

        # View it a second time, as the logic is different.
        self.client.get(post_1_url)

        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day1).count, 2)
        self.assertFalse(ViewCount.objects.filter(page=example_post_2.id).exists())

        # Now view the other page
        self.client.get(post_2_url)

        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day1).count, 2)
        self.assertEqual(ViewCount.objects.get(page=example_post_2.id, date=day1).count, 1)

        # And view the first page again
        self.client.get(post_1_url)

        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day1).count, 3)
        self.assertEqual(ViewCount.objects.get(page=example_post_2.id, date=day1).count, 1)

        mockdate.today.return_value = day2

        # And view the first page again
        self.client.get(post_1_url)

        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day1).count, 3)
        self.assertEqual(ViewCount.objects.get(page=example_post_2.id, date=day1).count, 1)
        self.assertEqual(ViewCount.objects.get(page=example_post_1.id, date=day2).count, 1)

    @mock.patch('info.views.date')
    def test_blog_sidebar_popular_posts(self, mockdate=None):
        from .views import BlogMixin

        example_post_1 = InfoPage.objects.create(
            slug="post",
            title="Example title",
            raw_content="Example content",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )

        example_post_2 = InfoPage.objects.create(
            slug="post2",
            title="Example title 2",
            raw_content="Example content 2",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )
        old_post = InfoPage.objects.create(
            slug="oldpost",
            title="Example title 3",
            raw_content="Example content 2",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
        )

        day1 = date(2015, 01, 01)

        ViewCount.objects.create(
            page=example_post_1,
            count=10,
            date=day1,
            )
        ViewCount.objects.create(
            page=example_post_2,
            count=5,
            date=day1,
            )
        ViewCount.objects.create(
            page=old_post,
            count=100,
            date=date(2000, 1, 1),
            )

        mockdate.today.return_value = day1

        context = BlogMixin().get_context_data()

        self.assertListEqual(
            [x['slug'] for x in context['popular_posts']],
            ['post', 'post2'],
            )

    @mock.patch('info.views.date')
    def test_blog_sidebar_popular_posts_published_last_month(self, mockdate=None):
        from .views import BlogMixin

        new_popular_post = InfoPage.objects.create(
            slug="new_popular_post",
            title="Example title 1",
            raw_content="Example content 1",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
            publication_date=date(2014, 12, 15),
        )
        new_unpopular_post = InfoPage.objects.create(
            slug="new_unpopular_post",
            title="Example title 2",
            raw_content="Example content 2",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
            publication_date=date(2014, 12, 20),
        )
        old_popular_post = InfoPage.objects.create(
            slug="old_popular_post",
            title="Example title 3",
            raw_content="Example content 3",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
            publication_date=date(2014, 1, 1),
        )
        old_popular_in_the_past_post = InfoPage.objects.create(
            slug="old_popular_in_the_past_post",
            title="Example title 4",
            raw_content="Example content 4",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
            publication_date=date(2014, 1, 1),
        )
        old_unpopular_post = InfoPage.objects.create(
            slug="old_unpopular_post",
            title="Example title 5",
            raw_content="Example content 5",
            use_raw=True,
            kind=InfoPage.KIND_BLOG,
            publication_date=date(2014, 1, 15),
        )

        fake_today = date(2015, 1, 1)

        ViewCount.objects.create(
            page=new_popular_post,
            count=100,
            date=new_popular_post.publication_date + timedelta(days=1),
            )
        ViewCount.objects.create(
            page=new_unpopular_post,
            count=5,
            date=new_unpopular_post.publication_date + timedelta(days=1),
            )
        ViewCount.objects.create(
            page=old_popular_post,
            count=500,
            date=date(2014, 12, 15),
            )
        ViewCount.objects.create(
            page=old_popular_in_the_past_post,
            count=600,
            date=old_popular_in_the_past_post.publication_date + timedelta(days=1),
            )
        ViewCount.objects.create(
            page=old_unpopular_post,
            count=1,
            date=old_unpopular_post.publication_date + timedelta(days=1),
            )

        mockdate.today.return_value = fake_today

        context = BlogMixin().get_context_data()

        self.assertListEqual(
            [x['slug'] for x in context['popular_recent_posts']],
            ['new_popular_post', 'new_unpopular_post'],
            )

class InfoBlogClientTests(TestCase):
    fixtures = ['sample_blog_posts.json']

    # def test_show_loaded_pages(self):
    #     pages = InfoPage.objects.all()
    #     if len(pages):
    #         for page in pages:
    #             print page
    #     else:
    #         print "no pages found"

    def _test_label(self, tests, url_base):
        c = Client()

        all_contents = list( itertools.chain( *tests.values() ) )
        # print all_contents

        for label, expected_contents in tests.items():
            url = url_base + label
            # print '------------------', url
            response = c.get(url)
            for content in expected_contents:
                # print label, "should contain", content
                self.assertContains(response, content)

            for content in all_contents:
                if content in expected_contents: continue
                # print label, "should not contain", content
                self.assertNotContains(response, content)

    def test_tags(self):
        self._test_label(
            tests = {
                "birds":   ["Blah Raven blah", "Blah Swan blah"],
                "mammals": ["Blah Black Panther blah", "Blah Polar Bear blah"],
                "birds,mammals": ["Blah Raven blah", "Blah Swan blah", "Blah Black Panther blah", "Blah Polar Bear blah"],
            },
            url_base = '/blog/tag/'
        )

    def test_categories(self):
        self._test_label(
            tests = {
                "category-1": ["Blah Raven blah", "Blah Polar Bear blah"],
                "category-2": ["Blah Black Panther blah", "Blah Swan blah"],
                "category-1,category-2": ["Blah Raven blah", "Blah Swan blah", "Blah Black Panther blah", "Blah Polar Bear blah"],
            },
            url_base = '/blog/category/'
        )

@override_settings(INFO_PAGES_ALLOW_RAW_HTML=True)
class ExtractFirstImageTests(TestCase):

    def setUp(self):
        storage = FileSystemStorage()
        with open(join(dirname(__file__), 'fixtures', 'bar.png'), 'rb') as f:
            storage_file = storage.save('file_archive/bar.png', f)
        self.uploaded_file = File.objects.create(
            slug='bar2',
            file=storage_file,
        )

    def tearDown(self):
        self.uploaded_file.delete()

    def test_extract_image_should_work(self):
        markdown = "Here's our nice graphic: " \
            "![Graphic](http://www.pa.org.za{0})".format(
                self.uploaded_file.file.url
            )
        example_infographic = InfoPage.objects.create(
            slug="fun-dataviz",
            title="Some fun data visualization",
            markdown_content=markdown,
            use_raw=False,
            kind=InfoPage.KIND_BLOG,
        )
        first_image = get_first_image_file(File, example_infographic)
        self.assertEqual(first_image.id, self.uploaded_file.id)

    def test_image_from_elsewhere(self):
        markdown = "Here's someone else's graphic: " \
            "![Another graphic](http://google.com/blah.png)"
        example_infographic = InfoPage.objects.create(
            slug="a-google-image",
            title="An image from elsewhere",
            markdown_content=markdown,
            use_raw=False,
            kind=InfoPage.KIND_BLOG,
        )
        first_image = get_first_image_file(File, example_infographic)
        self.assertIsNone(first_image)

    def test_extract_no_images(self):
        markdown = "There's no graphic here."
        example_infographic = InfoPage.objects.create(
            slug="boring-text",
            title="*Nothing* of interest here.",
            markdown_content=markdown,
            use_raw=False,
            kind=InfoPage.KIND_BLOG,
        )
        first_image = get_first_image_file(File, example_infographic)
        self.assertIsNone(first_image)

    def test_extract_image_not_found(self):
        markdown = "Here's our nice graphic: " \
            "![Graphic](http://www.pa.org.za/media_root/file_archive/quux.png)"
        example_infographic = InfoPage.objects.create(
            slug="fun-dataviz",
            title="An interesting visualization",
            markdown_content=markdown,
            use_raw=False,
            kind=InfoPage.KIND_BLOG,
        )
        first_image = get_first_image_file(File, example_infographic)
        self.assertIsNone(first_image)



import autocomplete_light.shortcuts as autocomplete_light

from .models import Category, Tag

autocomplete_light.register(Category, search_fields=['name'] )
autocomplete_light.register(Tag,      search_fields=['name'] )



from os.path import join
from urlparse import urlsplit

from bs4 import BeautifulSoup
from markdown import markdown

from django.conf import settings


def get_first_image_file(file_archive_file_class, info_page):
    """Return the first image in the post, if it's a file_archive image

    Otherwise (i.e. if there are no images or the first image is
    not one that's been uploaded to file_archive) return None.

    This is designed to be used from a migration, where you don't have
    the real InfoPage class, so we can't use .content_as_cleaned_html
    and instead have to use markdown to render the markdown to HTML."""
    if info_page.use_raw:
        html = info_page.raw_content
    else:
        html = markdown(unicode(info_page.markdown_content))
    soup = BeautifulSoup(html)
    imgs = soup.find_all('img')
    if not imgs:
        return None
    first_image_url = imgs[0]['src']
    # If this is an image uploaded to the file_archive, we expect
    # the URL to begin with the MEDIA_URL followed by 'file_archive':
    image_path = urlsplit(first_image_url).path
    media_url_path = urlsplit(settings.MEDIA_URL).path
    expected_prefix = join(media_url_path, 'file_archive')
    if not image_path.startswith(expected_prefix):
        return None
    path_after_media_url = image_path[len(media_url_path):]
    try:
        result = file_archive_file_class.objects.get(file=path_after_media_url)
    except file_archive_file_class.DoesNotExist:
        result = None
    return result



from django.conf import settings
from django.contrib import admin

import autocomplete_light.shortcuts as autocomplete_light

import models

from slug_helpers.admin import StricterSlugFieldMixin

@admin.register(models.Category, models.Tag)
class LabelAdmin(admin.ModelAdmin):
    search_fields = [ 'name' ]
    list_display  = [ 'name', 'slug' ]

    prepopulated_fields = {'slug': ['name']}

if settings.INFO_PAGES_ALLOW_RAW_HTML:
    search_fields_to_use = ['title', 'markdown_content', 'raw_content']
    fields_to_use = (
        'title', 'slug', 'publication_date', 'kind', 'use_raw',
        'markdown_content', 'raw_content', 'categories', 'tags',
        'featured_image_file',
    )
else:
    search_fields_to_use = ['title', 'markdown_content']
    fields_to_use = (
        'title', 'slug', 'publication_date', 'kind',
        'markdown_content', 'categories', 'tags',
        'featured_image_file',
    )

@admin.register(models.InfoPage)
class InfoPageAdmin(StricterSlugFieldMixin, admin.ModelAdmin):
    search_fields = search_fields_to_use
    list_display  = [ 'slug', 'title', 'kind', 'publication_date' ]
    list_filter   = [ 'kind', 'categories', 'tags' ]
    date_hierarchy = 'publication_date'
    ordering = ('-publication_date', 'title')

    form = autocomplete_light.modelform_factory(models.InfoPage, exclude=[])

    fields = fields_to_use
    prepopulated_fields = {'slug': ['title']}

    class Media:
        js = ("info/js/admin.js",)



from setuptools import setup, find_packages
setup(
    name = "django-info-pages",
    version = "0.0.4",
    packages = find_packages(),
    author = "Edmund von der Burg",
    author_email = "evdb@ecclestoad.co.uk",
    url = "https://github.com/mysociety/django-info-pages",
    description = "Editable info pages and basic blog functionality for Django",
    license = "AGPL",
    keywords = "django blog cms",
    include_package_data=True,
    install_requires = [
        'Django>=1.8.12,<1.9',
        'beautifulsoup4',
        'lxml',
        'Markdown',
        'django-markitup',
        'nose',
        'django-autocomplete-light==2.3.3',
        'django-file-archive==0.0.2',
        'django-pagination==1.0.7',
        'django-slug-helpers>=0.0.3',
    ],
    extras_require={
        'test': [
            'mock',
        ],
    },
)



from django.conf.urls import patterns, url, handler404

from ..views import InfoPageView


urlpatterns = patterns('',
    url(r'^$',                  InfoPageView.as_view(), { 'slug': 'index' } ),
    url(r'^(?P<slug>[\w\-]+)$', InfoPageView.as_view(), name='info_page' ),
    url(r'^.*$',                handler404    ),
)



from datetime import date, timedelta

from django.views.generic import DetailView, ListView
from django.views.generic.base import ContextMixin
from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse_lazy
from django.db.models import F, Sum
from django.shortcuts import get_list_or_404
from django.conf import settings

from models import InfoPage, Category, Tag, ViewCount


class BlogMixin(ContextMixin):

    def get_context_data(self, **kwargs):
        context = super(BlogMixin, self).get_context_data(**kwargs)

        context['all_categories'] = Category.objects.all().order_by('name')

        context['recent_posts'] = InfoPage.objects \
            .filter(kind=InfoPage.KIND_BLOG) \
            .order_by("-publication_date")

        context['some_popular_posts'] = viewcounts = ViewCount.objects.filter(
            page__kind=InfoPage.KIND_BLOG
        ).count()

        if viewcounts:
            # We randomize the order of blog posts with the same number
            # of views - this is only really useful when we have
            # very little data, since with more data it's unlikely
            # there will be many pages with the same number of view
            # counts in the recent period anyway
            randomize_same_sum = viewcounts < 20
            context['popular_posts'] = self.popular_posts_queryset(
                only_recent_posts=False,
                randomize_same_sum=randomize_same_sum)
            context['popular_recent_posts'] = self.popular_posts_queryset(
                only_recent_posts=True,
                randomize_same_sum=randomize_same_sum)

        return context

    def popular_posts_queryset(self, only_recent_posts, randomize_same_sum):
        date_condition = {}
        if only_recent_posts:
            date_condition = {
                'publication_date__gte': date.today() - timedelta(days=28)}

        # The '?' means to randomize the order among counts that
        # are the same
        if randomize_same_sum:
            order_args = ('-viewcount__count__sum', '?')
        else:
            order_args = ('-viewcount__count__sum',)

        return (InfoPage.objects
            .filter(kind=InfoPage.KIND_BLOG)
            .filter(**date_condition)
            .filter(viewcount__count__gt=0)
            .filter(viewcount__date__gte=date.today() - timedelta(days=28))
            .values('title', 'slug')
            .annotate(Sum('viewcount__count'))
            .order_by(*order_args)
        )

class InfoBlogList(BlogMixin, ListView):
    """Show list of blog posts"""
    model = InfoPage
    queryset = InfoPage.objects.filter(kind=InfoPage.KIND_BLOG).order_by("-publication_date")
    paginate_by = settings.INFO_POSTS_PER_LIST_PAGE
    template_name = 'info/blog_list.html'


class InfoBlogLabelBase(InfoBlogList):

    def get_queryset(self):
        slugs = self.kwargs['slug'].split(',')
        queryset = super(InfoBlogLabelBase, self).get_queryset()
        filter_args = { self.filter_field: slugs }
        return queryset.filter(**filter_args)

    def get_context_data(self, **kwargs):
        context = super(InfoBlogLabelBase, self).get_context_data(**kwargs)

        slugs = self.kwargs['slug'].split(',')
        context[self.context_key] = get_list_or_404(self.context_filter_model, slug__in=slugs)

        return context


class InfoBlogCategory(InfoBlogLabelBase):
    context_key  = 'categories'
    context_filter_model = Category
    filter_field = 'categories__slug__in'

    def get_context_data(self, **kwargs):
        context = super(InfoBlogCategory, self).get_context_data(**kwargs)

        # Filter the recent posts to be specific to this category
        slugs = self.kwargs['slug'].split(',')
        context['recent_posts'] = context['recent_posts'].filter(categories__slug__in=slugs)

        context['category_names'] = []
        for category in context['categories']:
            context['category_names'].append(category.name)

        return context


class InfoBlogTag(InfoBlogLabelBase):
    context_key  = 'tags'
    context_filter_model = Tag
    filter_field = 'tags__slug__in'


class InfoBlogView(BlogMixin, DetailView):
    """Show the blog post for the given slug"""
    model = InfoPage
    queryset = InfoPage.objects.filter(kind=InfoPage.KIND_BLOG)
    template_name = 'info/blog_post.html'

    def get(self, request, *args, **kwargs):
        response = super(InfoBlogView, self).get(request, *args, **kwargs)

        _, created = ViewCount.objects.get_or_create(
            page=self.object,
            date=date.today(),
            defaults={'count': 1},
            )

        if not created:
            (ViewCount.objects.filter(page=self.object, date=date.today())
             .update(count=F('count')+1))

        return response


class InfoBlogFeed(Feed):
    """Create a feed with the latest 10 blog entries in"""
    title = "Recent blog posts"
    link = reverse_lazy('info_blog_list')
    description = "Recent blog posts"

    def items(self):
        return InfoPage.objects.filter(kind=InfoPage.KIND_BLOG).order_by("-publication_date")[:10]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.content_as_cleaned_html


class InfoPageView(DetailView):
    """Show the page for the given slug"""
    model = InfoPage
    queryset = InfoPage.objects.filter(kind=InfoPage.KIND_PAGE)
